@@ -1,6 +1,10 @@
-from ._version import __version__  # noqa: F401
+"""Async Task Queue Orchestrator with Complex Resource Management"""
+
 from .bases import ActorBase, DistAPIBase, TaskPropertyBase  # noqa: F401
 from .core import Scheduler, SchedulerTask  # noqa: F401
+from .distributed_apis import DEFAULT_DIST_API_KEY, DEFAULT_MULTI_API  # noqa: F401
 from .exceptions import UnexpectedCapabilities  # noqa: F401
 from .resource_handling import Capability, CapabilitySet  # noqa: F401
 from .simplified_functions import parallel_map  # noqa: F401
+
+__version__ = "0.1.6"
