@@ -1,29 +1,21 @@
 import asyncio
 import itertools
 import uuid
+from dataclasses import dataclass
 from enum import Enum
 from itertools import chain
 from queue import Empty, Queue
 from threading import Thread
-from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, List, Type
+from typing import Any, Callable, Dict, Iterable, List, Type
 
 from structlog import get_logger
 
-from atqo.bases import TaskPropertyBase
-
+from .bases import ActorBase, DistAPIBase, TaskPropertyBase
 from .distributed_apis import DEFAULT_DIST_API_KEY, get_dist_api
-from .exceptions import (
-    ActorListenBreaker,
-    ActorPoisoned,
-    NotEnoughResourcesToContinue,
-)
+from .exceptions import ActorListenBreaker, ActorPoisoned, NotEnoughResourcesToContinue
 from .exchange import CapsetExchange
 from .resource_handling import Capability, CapabilitySet, NumStore
 
-if TYPE_CHECKING:
-    from .bases import ActorBase, DistAPIBase  # pragma: no cover
-
-
 POISON_KEY = frozenset([])  # just make sure it comes before any other
 POISON_PILL = None
 ALLOWED_CONSUMER_FAILS = 5
@@ -34,13 +26,18 @@ def _start_loop(loop: asyncio.AbstractEventLoop) -> None:
     loop.run_forever()
 
 
+def _get_loop_of_daemon():
+    loop = asyncio.new_event_loop()
+    Thread(target=_start_loop, args=(loop,), daemon=True).start()
+    return loop
+
+
 class Scheduler:
     def __init__(
         self,
         actor_dict: Dict[CapabilitySet, Type["ActorBase"]],
         resource_limits: Dict[Enum, float],
         distributed_system: str = DEFAULT_DIST_API_KEY,
-        reorganize_after_every_task: bool = False,  # overkill
         verbose=False,
     ) -> None:
         """Core scheduler class
@@ -51,30 +48,25 @@ class Scheduler:
           -
 
         """
+        for _, v in actor_dict.items():
+            assert ActorBase in v.mro()
 
         self._result_queue = Queue()
         self._active_async_tasks = set()
         self._task_queues: Dict[CapabilitySet, TaskQueue] = {}
+        self._loop = _get_loop_of_daemon()
 
-        self._loop = asyncio.new_event_loop()
-
-        Thread(target=_start_loop, args=(self._loop,), daemon=True).start()
-
-        self._frequent_reorg = reorganize_after_every_task
         self._verbose = verbose
         self._dist_api: DistAPIBase = get_dist_api(distributed_system)()
 
         self._actor_sets: Dict[CapabilitySet, ActorSet] = {}
         self._run(self._add_actor_sets(actor_dict))
-
-        self._capset_exchange = CapsetExchange(
-            actor_dict.keys(), resource_limits
-        )
+        self._capset_exchange = CapsetExchange(actor_dict.keys(), resource_limits)
 
         # TODO
         # concurrent_task_limit: Callable[[List[TaskPropertyBase]], bool]
         # self._active_task_properties = ActiveTaskPropertySet()
-        # self._task_limiter = concurrent_task_limit  # TODO
+        # self._task_limiter = concurrent_task_limit
 
     def __del__(self):
         try:
@@ -95,7 +87,8 @@ class Scheduler:
             next_batch = batch_producer()
             batch_size = len(next_batch)
             empty_batch = batch_size == 0
-            self._log("new batch", size=batch_size, was_done=is_done)
+            logstr = f"{'empty' if empty_batch else 'new'} batch"
+            self._log(logstr, size=batch_size, was_done=is_done)
             if is_done and empty_batch:
                 break
             if empty_batch:
@@ -157,7 +150,6 @@ class Scheduler:
             ).info(logstr, **kwargs)
 
     def _q_of_new_capset(self, capset: CapabilitySet) -> asyncio.Queue:
-
         new_task_queue = TaskQueue()
         self._task_queues[capset] = new_task_queue
         for task_cs, task_queue in self._task_queues.items():
@@ -194,8 +186,10 @@ class Scheduler:
         self, scheduler_task: "SchedulerTask"
     ):
         scheduler_task.init_future()
-        task_result = await scheduler_task.future
-        self._result_queue.put(task_result)
+        task_result: TaskResult = await scheduler_task.future
+        self._result_queue.put(task_result.value)
+        if task_result.is_last_in_queue and self.queued_task_count:
+            await self._reorganize_actors()
 
     async def _reorganize_actors(self):
         """optimize actor set sizes
@@ -228,19 +222,12 @@ class Scheduler:
             )
 
     async def _await_until(self, remaining_tasks: int = 0):
-        return_when = (
-            "FIRST_COMPLETED"
-            if (self._frequent_reorg or remaining_tasks) > 0
-            else "ALL_COMPLETED"
-        )
+        return_when = "FIRST_COMPLETED" if remaining_tasks > 0 else "ALL_COMPLETED"
         while len(self._active_async_tasks) > remaining_tasks:
             done, _ = await asyncio.wait(
                 self._active_async_tasks, return_when=return_when
             )
             self._active_async_tasks.difference_update(done)
-            if self._frequent_reorg:
-                await self._reorganize_actors()
-
         await self._reorganize_actors()
 
     async def _drain_all_actor_sets(self):
@@ -259,9 +246,7 @@ class Scheduler:
 
     @property
     def _running_consumer_count(self):
-        return sum(
-            [aset.running_actor_count for aset in self._actor_sets.values()]
-        )
+        return sum([aset.running_actor_count for aset in self._actor_sets.values()])
 
     @property
     def _all_actors(self):
@@ -322,8 +307,6 @@ class ActorSet:
         self._actor_listening_async_task_dict: Dict[str, asyncio.Task] = {}
         self._debug = debug
 
-        self._async_queue_get_task_dict = ...
-
     def __repr__(self):
         dic_str = [f"{k}={v}" for k, v in self._log_dic.items()]
         return f"{type(self).__name__}({', '.join(dic_str)}"
@@ -345,16 +328,14 @@ class ActorSet:
         return n
 
     async def add_new_actor(self):
-        running_actor = self.dist_api.get_running_actor(
-            actor_cls=self.actor_cls
-        )
+        running_actor = self.dist_api.get_running_actor(actor_cls=self.actor_cls)
         listener_name = uuid.uuid1().hex
         coroutine = self._listen(
             running_actor=running_actor,
             name=listener_name,
         )
         task = asyncio.create_task(coroutine, name=listener_name)
-        self._logger.info("adding consumer", listener_task=task.get_name())
+        self._log("adding consumer", listener_task=task.get_name())
         self._actor_listening_async_task_dict[listener_name] = task
 
     @property
@@ -370,28 +351,24 @@ class ActorSet:
         return self._actor_listening_async_task_dict.values()
 
     async def _listen(self, running_actor: "ActorBase", name: str):
-        self._logger.info(
+        # WARNING: if error happens here, it will get swallowed to the abyss
+        self._log(
             "consumer listening",
             running=type(running_actor).__name__,
+            ra=running_actor.restart_after,
         )
         fails = 0
+        runs = 0
         while True:
             next_task = await self._get_next_task()
             try:
-                fails = await self._process_task(
-                    running_actor, next_task, fails
-                )
+                fails = await self._process_task(running_actor, next_task, fails)
+                runs += 1
             except ActorListenBreaker as e:
-                self._logger.info(
-                    "stopping consumer",
-                    reason=e,
-                    running=type(running_actor).__name__,
-                )
-                self.dist_api.kill(running_actor)
-                del self._actor_listening_async_task_dict[name]
-                self._poisoning_done_future.set_result(True)
-                if not isinstance(e, ActorPoisoned):
-                    await self.add_new_actor()
+                await self._end_actor(running_actor, e, name)
+                return
+            if runs >= running_actor.restart_after:
+                await self._end_actor(running_actor, None, name)
                 return
 
     async def _get_next_task(self) -> "SchedulerTask":
@@ -416,25 +393,46 @@ class ActorSet:
             out = await self.dist_api.get_future(running_actor, next_task)
             if isinstance(out, Exception):
                 raise out
-            next_task.set_future(out)
+            result = TaskResult(out, True, self._is_last(next_task))
+            next_task.set_future(result)
             return 0
         except self.dist_api.exception as e:
-            self._logger.warning(
-                "Remote consumption error ",
-                e=e,
-                te=type(e),
-            )
+            self._log("Remote consumption error ", e=e, te=type(e))
             if self._debug:
                 self._logger.exception(e)
             next_task.fail_count += 1
             if next_task.fail_count > next_task.max_fails:
-                next_task.set_future(self.dist_api.parse_exception(e))
+                is_last = self._is_last(next_task)
+                result = TaskResult(self.dist_api.parse_exception(e), False, is_last)
+                next_task.set_future(result)
             else:
                 await self._task_queues[next_task.requirements].put(next_task)
         if fails >= ALLOWED_CONSUMER_FAILS:
             raise ActorListenBreaker(f"{fails} number of fails reached")
         return fails + 1
 
+    async def _end_actor(self, running_actor: "ActorBase", e, name):
+        self._log(
+            "stopping consumer",
+            reason=e,
+            running=type(running_actor).__name__,
+            ra=running_actor.restart_after,
+        )
+        self.dist_api.kill(running_actor)
+        del self._actor_listening_async_task_dict[name]
+        if not isinstance(e, ActorPoisoned):
+            self._log("restarting consumer")
+            await self.add_new_actor()
+        else:
+            self._poisoning_done_future.set_result(True)
+
+    def _log(self, s, **kwargs):
+        if self._debug:
+            self._logger.info(s, **kwargs)
+
+    def _is_last(self, task: "SchedulerTask"):
+        return self._task_queues[task.requirements].size == 0
+
     @property
     def _wait_on_tasks(self):
         return chain(
@@ -479,8 +477,22 @@ class SchedulerTask:
         self.fail_count = 0
         self.future = None
 
+    def __repr__(self) -> str:
+        return (
+            f"Task: {self.argument}, "
+            f"Requires: {self.requirements}, "
+            f"Future: {self.future}"
+        )
+
     def init_future(self):
         self.future = asyncio.Future()
 
-    def set_future(self, value):
-        self.future.set_result(value)
+    def set_future(self, task_result):
+        self.future.set_result(task_result)
+
+
+@dataclass
+class TaskResult:
+    value: Any
+    is_ok: bool
+    is_last_in_queue: bool
