@@ -4,13 +4,13 @@ from multiprocessing import cpu_count
 
 from .bases import ActorBase
 from .core import Scheduler, SchedulerTask
-from .distributed_apis import DEFAULT_DIST_API_KEY
+from .distributed_apis import DEFAULT_MULTI_API
 from .resource_handling import Capability, CapabilitySet
-from .utils import partial_wrap
+from .utils import partial_cls
 
 _RES = "CPU"
 _CAP = Capability({_RES: 1})
-_Task = partial_wrap(SchedulerTask, requirements=[_CAP])
+_Task = partial_cls(SchedulerTask, requirements=[_CAP])
 
 
 class BatchProd:
@@ -31,9 +31,11 @@ class ActWrap(ActorBase):
         return self._f(task_arg)
 
 
-def get_simp_scheduler(n, fun, dist_sys, verbose) -> Scheduler:
+def get_simp_scheduler(n, fun, dist_sys, verbose, max_runs) -> Scheduler:
+    ActWrap.restart_after = max_runs
+
     return Scheduler(
-        actor_dict={CapabilitySet([_CAP]): partial_wrap(ActWrap, fun=fun)},
+        actor_dict={CapabilitySet([_CAP]): partial_cls(ActWrap, fun=fun)},
         resource_limits={_RES: n},
         distributed_system=dist_sys,
         verbose=verbose,
@@ -43,13 +45,14 @@ def get_simp_scheduler(n, fun, dist_sys, verbose) -> Scheduler:
 def parallel_map(
     fun,
     iterable,
-    dist_api=DEFAULT_DIST_API_KEY,
+    dist_api=DEFAULT_MULTI_API,
     batch_size=None,
     min_queue_size=None,
     workers=None,
-    raise_errors=False,
+    raise_errors=True,
     verbose=False,
     pbar=False,
+    restart_after=float("inf"),
 ):
     nw = workers or cpu_count()
     batch_size = batch_size or nw * 5
@@ -60,7 +63,7 @@ def parallel_map(
 
         iterable = tqdm(iterable)
 
-    scheduler = get_simp_scheduler(nw, fun, dist_api, verbose)
+    scheduler = get_simp_scheduler(nw, fun, dist_api, verbose, restart_after)
 
     out = []
     outmap = partial(_raiser, li=out) if raise_errors else out.append
