@@ -1,5 +1,6 @@
 import pytest
 
+from atqo.bases import ActorBase
 from atqo.core import Scheduler, SchedulerTask
 from atqo.exceptions import NotEnoughResources
 from atqo.resource_handling import Capability, CapabilitySet
@@ -17,9 +18,7 @@ def test_dead_end():
     cap1 = Capability({"A": 1})
     cap2 = Capability({"A": 2})
 
-    scheduler = Scheduler({CapabilitySet([cap1]): None}, {"A": 1})
+    scheduler = Scheduler({CapabilitySet([cap1]): ActorBase}, {"A": 1})
 
     with pytest.raises(NotEnoughResources):
-        scheduler.refill_task_queue(
-            [SchedulerTask("x", requirements=[cap1, cap2])]
-        )
+        scheduler.refill_task_queue([SchedulerTask("x", requirements=[cap1, cap2])])
